@@ -77,11 +77,11 @@ std::vector<Sub> Vars(const FieldDescriptor* field, const Options& opts,
 
 class SingularMessage : public FieldGeneratorBase {
  public:
-  SingularMessage(const FieldDescriptor* field, const Options& opts,
-                  MessageSCCAnalyzer* scc)
-      : FieldGeneratorBase(field, opts, scc),
+  SingularMessage(const FieldDescriptor* field, const Options& opts)
+      : FieldGeneratorBase(field, opts),
         opts_(&opts),
-        has_required_(scc->HasRequiredFields(field->message_type())),
+        has_required_(
+            opts.scc_analyzer->HasRequiredFields(field->message_type())),
         has_hasbit_(HasHasbit(field, opts)) {}
 
   ~SingularMessage() override = default;
@@ -107,7 +107,6 @@ class SingularMessage : public FieldGeneratorBase {
   void GenerateMergingCode(io::Printer* p) const override;
   void GenerateSwappingCode(io::Printer* p) const override;
   void GenerateDestructorCode(io::Printer* p) const override;
-  void GenerateConstructorCode(io::Printer* p) const override {}
   void GenerateCopyConstructorCode(io::Printer* p) const override;
   void GenerateSerializeWithCachedSizesToArray(io::Printer* p) const override;
   void GenerateByteSize(io::Printer* p) const override;
@@ -157,7 +156,7 @@ void SingularMessage::GenerateAccessorDeclarations(io::Printer* p) const {
       AnnotatedAccessors(field_, {"mutable_"}, AnnotationCollector::kAlias));
 
   p->Emit(R"cc(
-    $DEPRECATED$ const $Submsg$& $name$() const;
+    $DEPRECATED$ [[nodiscard]] const $Submsg$& $name$() const;
     $DEPRECATED$ [[nodiscard]] $Submsg$* $nullable$ $release_name$();
     $DEPRECATED$ $Submsg$* $nonnull$ $mutable_name$();
     $DEPRECATED$ void $set_allocated_name$($Submsg$* $nullable$ value);
@@ -379,7 +378,7 @@ void SingularMessage::GenerateDestructorCode(io::Printer* p) const {
 void SingularMessage::GenerateCopyConstructorCode(io::Printer* p) const {
   ABSL_CHECK(has_hasbit_);
   p->Emit(R"cc(
-    if ((from.$has_hasbit$) != 0) {
+    if (CheckHasBit(from.$has_bits_array$, $has_mask$)) {
       _this->$field_$ = $superclass$::CopyConstruct(arena, *from.$field_$);
     }
   )cc");
@@ -415,7 +414,7 @@ void SingularMessage::GenerateIsInitialized(io::Printer* p) const {
 
   if (HasHasbit(field_, *opts_)) {
     p->Emit(R"cc(
-      if ((this_.$has_hasbit$) != 0) {
+      if (CheckHasBit(this_.$has_bits_array$, $has_mask$)) {
         if (!this_.$field_$->IsInitialized()) return false;
       }
     )cc");
@@ -457,9 +456,8 @@ void SingularMessage::GenerateAggregateInitializer(io::Printer* p) const {
 
 class OneofMessage : public SingularMessage {
  public:
-  OneofMessage(const FieldDescriptor* descriptor, const Options& options,
-               MessageSCCAnalyzer* scc_analyzer)
-      : SingularMessage(descriptor, options, scc_analyzer) {
+  OneofMessage(const FieldDescriptor* descriptor, const Options& options)
+      : SingularMessage(descriptor, options) {
     auto* oneof = descriptor->containing_oneof();
     num_message_fields_in_oneof_ = 0;
     for (int i = 0; i < oneof->field_count(); ++i) {
@@ -503,7 +501,6 @@ class OneofMessage : public SingularMessage {
   void GenerateMessageClearingCode(io::Printer* p) const override;
   void GenerateSwappingCode(io::Printer* p) const override;
   void GenerateDestructorCode(io::Printer* p) const override;
-  void GenerateConstructorCode(io::Printer* p) const override;
   void GenerateCopyConstructorCode(io::Printer* p) const override;
   void GenerateIsInitialized(io::Printer* p) const override;
   bool NeedsIsInitialized() const override;
@@ -560,7 +557,8 @@ void OneofMessage::GenerateInlineAccessorDefinitions(io::Printer* p) const {
   p->Emit(R"cc(
     inline const $Submsg$& $Msg$::_internal_$name_internal$() const {
       $StrongRef$;
-      return $has_field$ ? *$cast_field_$ : reinterpret_cast<$Submsg$&>($kDefault$);
+      return $has_field$ ? static_cast<const $Submsg$&>(*$cast_field_$)
+                         : reinterpret_cast<const $Submsg$&>($kDefault$);
     }
   )cc");
   p->Emit(R"cc(
@@ -664,11 +662,6 @@ void OneofMessage::GenerateDestructorCode(io::Printer* p) const {
   // behavior.
 }
 
-void OneofMessage::GenerateConstructorCode(io::Printer* p) const {
-  // Don't print any constructor code. The field is in a union. We allocate
-  // space only when this field is used.
-}
-
 void OneofMessage::GenerateCopyConstructorCode(io::Printer* p) const {
   ABSL_CHECK(!has_hasbit_);
   p->Emit(R"cc(
@@ -713,11 +706,11 @@ bool OneofMessage::RequiresArena(GeneratorFunction func) const {
 
 class RepeatedMessage : public FieldGeneratorBase {
  public:
-  RepeatedMessage(const FieldDescriptor* field, const Options& opts,
-                  MessageSCCAnalyzer* scc)
-      : FieldGeneratorBase(field, opts, scc),
+  RepeatedMessage(const FieldDescriptor* field, const Options& opts)
+      : FieldGeneratorBase(field, opts),
         opts_(&opts),
-        has_required_(scc->HasRequiredFields(field->message_type())) {}
+        has_required_(
+            opts.scc_analyzer->HasRequiredFields(field->message_type())) {}
 
   ~RepeatedMessage() override = default;
 
@@ -731,7 +724,6 @@ class RepeatedMessage : public FieldGeneratorBase {
   void GenerateClearingCode(io::Printer* p) const override;
   void GenerateMergingCode(io::Printer* p) const override;
   void GenerateSwappingCode(io::Printer* p) const override;
-  void GenerateConstructorCode(io::Printer* p) const override;
   void GenerateCopyConstructorCode(io::Printer* p) const override;
   void GenerateDestructorCode(io::Printer* p) const override;
   void GenerateSerializeWithCachedSizesToArray(io::Printer* p) const override;
@@ -739,6 +731,8 @@ class RepeatedMessage : public FieldGeneratorBase {
   void GenerateIsInitialized(io::Printer* p) const override;
   bool NeedsIsInitialized() const override;
 
+  bool RequiresArena(GeneratorFunction function) const override;
+
  private:
   const Options* opts_;
   bool has_required_;
@@ -763,8 +757,9 @@ void RepeatedMessage::GenerateAccessorDeclarations(io::Printer* p) const {
                                            io::AnnotationCollector::kAlias));
 
   p->Emit(R"cc(
-    $DEPRECATED$ $Submsg$* $nonnull$ $mutable_name$(int index);
-    $DEPRECATED$ $pb$::RepeatedPtrField<$Submsg$>* $nonnull$ $mutable_name$();
+    [[nodiscard]] $DEPRECATED$ $Submsg$* $nonnull$ $mutable_name$(int index);
+    [[nodiscard]] $DEPRECATED$ $pb$::RepeatedPtrField<$Submsg$>* $nonnull$
+    $mutable_name$();
 
     private:
     const $pb$::RepeatedPtrField<$Submsg$>& $_internal_name$() const;
@@ -778,16 +773,21 @@ void RepeatedMessage::GenerateAccessorDeclarations(io::Printer* p) const {
   }
   p->Emit(R"cc(
     public:
-    $DEPRECATED$ const $Submsg$& $name$(int index) const;
+    [[nodiscard]] $DEPRECATED$ const $Submsg$& $name$(int index) const;
     $DEPRECATED$ $Submsg$* $nonnull$ $add_name$();
-    $DEPRECATED$ const $pb$::RepeatedPtrField<$Submsg$>& $name$() const;
+    [[nodiscard]] $DEPRECATED$ const $pb$::RepeatedPtrField<$Submsg$>& $name$()
+        const;
   )cc");
 }
 
 void RepeatedMessage::GenerateInlineAccessorDefinitions(io::Printer* p) const {
   // TODO: move insertion points
+
   p->Emit({GetEmitRepeatedFieldMutableSub(*opts_, p)},
           R"cc(
+            //~ Note: no need to set hasbit in mutable_$name$(int index).
+            //~ Hasbits only need to be updated if a new element is
+            //~ (potentially) added, not if an existing element is mutated.
             inline $Submsg$* $nonnull$ $Msg$::mutable_$name$(int index)
                 ABSL_ATTRIBUTE_LIFETIME_BOUND {
               $WeakDescriptorSelfPin$;
@@ -802,6 +802,7 @@ void RepeatedMessage::GenerateInlineAccessorDefinitions(io::Printer* p) const {
     inline $pb$::RepeatedPtrField<$Submsg$>* $nonnull$ $Msg$::mutable_$name$()
         ABSL_ATTRIBUTE_LIFETIME_BOUND {
       $WeakDescriptorSelfPin$;
+      $set_hasbit$;
       $annotate_mutable_list$;
       // @@protoc_insertion_point(field_mutable_list:$pkg.Msg.field$)
       $StrongRef$;
@@ -825,7 +826,10 @@ void RepeatedMessage::GenerateInlineAccessorDefinitions(io::Printer* p) const {
         ABSL_ATTRIBUTE_LIFETIME_BOUND {
       $WeakDescriptorSelfPin$;
       $TsanDetectConcurrentMutation$;
-      $Submsg$* _add = _internal_mutable_$name_internal$()->Add();
+      $Submsg$* _add =
+          _internal_mutable_$name_internal$()->InternalAddWithArena(
+              $pb$::MessageLite::internal_visibility(), GetArena());
+      $set_hasbit$;
       $annotate_add_mutable$;
       // @@protoc_insertion_point(field_add:$pkg.Msg.field$)
       return _add;
@@ -901,7 +905,8 @@ void RepeatedMessage::GenerateMergingCode(io::Printer* p) const {
   // `if (!from.empty()) { body(); }` for both split and non-split cases.
   auto body = [&] {
     p->Emit(R"cc(
-      _this->_internal_mutable$_weak$_$name$()->MergeFrom(
+      _this->_internal_mutable$_weak$_$name$()->InternalMergeFromWithArena(
+          $pb$::MessageLite::internal_visibility(), arena,
           from._internal$_weak$_$name$());
     )cc");
   };
@@ -923,17 +928,15 @@ void RepeatedMessage::GenerateSwappingCode(io::Printer* p) const {
   )cc");
 }
 
-void RepeatedMessage::GenerateConstructorCode(io::Printer* p) const {
-  // Not needed for repeated fields.
-}
-
 void RepeatedMessage::GenerateCopyConstructorCode(io::Printer* p) const {
   // TODO: For split repeated fields we might want to use type
   // erasure to reduce binary size costs.
   if (should_split()) {
     p->Emit(R"cc(
       if (!from._internal$_weak$_$name$().empty()) {
-        _internal_mutable$_weak$_$name$()->MergeFrom(from._internal$_weak$_$name$());
+        _internal_mutable$_weak$_$name$()->InternalMergeFromWithArena(
+            $pb$::MessageLite::internal_visibility(), arena,
+            from._internal$_weak$_$name$());
       }
     )cc");
   }
@@ -1040,24 +1043,30 @@ void RepeatedMessage::GenerateIsInitialized(io::Printer* p) const {
 }
 
 bool RepeatedMessage::NeedsIsInitialized() const { return has_required_; }
+
+bool RepeatedMessage::RequiresArena(GeneratorFunction func) const {
+  switch (func) {
+    case GeneratorFunction::kMergeFrom:
+      return true;
+  }
+  return false;
+}
+
 }  // namespace
 
 std::unique_ptr<FieldGeneratorBase> MakeSinguarMessageGenerator(
-    const FieldDescriptor* desc, const Options& options,
-    MessageSCCAnalyzer* scc) {
-  return absl::make_unique<SingularMessage>(desc, options, scc);
+    const FieldDescriptor* desc, const Options& options) {
+  return absl::make_unique<SingularMessage>(desc, options);
 }
 
 std::unique_ptr<FieldGeneratorBase> MakeRepeatedMessageGenerator(
-    const FieldDescriptor* desc, const Options& options,
-    MessageSCCAnalyzer* scc) {
-  return absl::make_unique<RepeatedMessage>(desc, options, scc);
+    const FieldDescriptor* desc, const Options& options) {
+  return absl::make_unique<RepeatedMessage>(desc, options);
 }
 
 std::unique_ptr<FieldGeneratorBase> MakeOneofMessageGenerator(
-    const FieldDescriptor* desc, const Options& options,
-    MessageSCCAnalyzer* scc) {
-  return absl::make_unique<OneofMessage>(desc, options, scc);
+    const FieldDescriptor* desc, const Options& options) {
+  return absl::make_unique<OneofMessage>(desc, options);
 }
 
 }  // namespace cpp
